/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapdb.h>
#include <Map/mapfclass.h>
#include <Map/maplib.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>
#include <IV-look/kit.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include <stdio.h>

ClassId MapLibCmd::GetClassId () { return MAPLIB_CMD; }

boolean MapLibCmd::IsA (ClassId id) {
    return MAPLIB_CMD == id || Command::IsA(id);
}

MapLibCmd::MapLibCmd (ControlInfo* c, const char* n)
: Command(c)
{
    name = (char*)n;
}

MapLibCmd::MapLibCmd (Editor* ed, const char* n)
: Command(ed)
{
    name = (char*)n;
}

void MapLibCmd::Init() {
    _list = new StringList;
    String* str_i;
    MapEditor* editor = (MapEditor*)GetEditor();
    MapDatabase* mapdb = editor->GetMapDatabase();
    int nlibs = mapdb->nlibs();
    for (int i=0; i<nlibs; i++) {
	str_i = new String(mapdb->library(i)->name());
	_list->append(*str_i);
    }
    _chooser = new StrChooser(_list, new String("Select MAP Library:"),
			      WidgetKit::instance(),
			      Session::instance()->style());
    Resource::ref(_chooser);
}

void MapLibCmd::Clean() {
    delete _list;
    Resource::unref(_chooser);
}

Command* MapLibCmd::Copy () {
    Command* copy = new MapLibCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapLibCmd::Execute () {
    if (!name) {
	Init();
	MapEditor* editor = (MapEditor*)GetEditor();
	while (_chooser->post_for(editor->GetWindow())) {
	    String str;
	    if (_chooser->selected() > -1)
		str = _list->item_ref(_chooser->selected());
	    else
		continue;
	    const char* nam = str.string();
	    editor->libstate()->maplib(nam);
	    break;
	}
	Clean();
    }
    else
	((MapEditor*)GetEditor())->libstate()->maplib(name);
    ((MapEditor*)GetEditor())->UpdateLibraryMenu();
}

boolean MapLibCmd::Reversible () { return false; }

/******************************************************************************/

ClassId MapCovCmd::GetClassId () { return MAPCOV_CMD; }

boolean MapCovCmd::IsA (ClassId id) {
    return MAPCOV_CMD == id || Command::IsA(id);
}

MapCovCmd::MapCovCmd (ControlInfo* c) : Command(c) { }
MapCovCmd::MapCovCmd (Editor* ed) : Command(ed) { }

void MapCovCmd::Init() {
    _namelist = new StringList;
    _desclist = new StringList;
    String* str_i;
    MapEditor* editor = (MapEditor*)GetEditor();
    MapLibState* libstate = editor->libstate();
    if (libstate->maplib() && strlen(libstate->maplib())>0) {
	MapLibrary* maplib = (MapLibrary*)editor->GetMapDatabase()->library(libstate->maplib());
	int ncovs = maplib->ncovs();
	for (int i=0; i<ncovs; i++) {
	    str_i = new String(maplib->coverage(i)->name());
	    _namelist->append(*str_i);
	    const char* desc = maplib->coverage(i)->description();
	    if (desc) {
		str_i = new String(desc);
		_desclist->append(*str_i);
	    }
	}
    }
    _chooser = new StrChooser(_desclist, new String("Select MAP Coverage:"),
			      WidgetKit::instance(),
			      Session::instance()->style());
    Resource::ref(_chooser);
}

void MapCovCmd::Clean() {
    delete _namelist;
    delete _desclist;
    Resource::unref(_chooser);
}

Command* MapCovCmd::Copy () {
    Command* copy = new MapCovCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapCovCmd::Execute () {
    Init();
    MapEditor* editor = (MapEditor*)GetEditor();
    while (_chooser->post_for(editor->GetWindow())) {
        String str;
	String dstr;
        if (_chooser->selected() > -1) {
            str = _namelist->item_ref(_chooser->selected());
	    dstr = _desclist->item_ref(_chooser->selected());
	}
	else
	    continue;
        const char* name = str.string();
	const char* desc = dstr.string();
	editor->covstate()->mapcov(name,desc);
	break;
    }
    Clean();
}

boolean MapCovCmd::Reversible () { return false; }

/******************************************************************************/

ClassId MapFClassCmd::GetClassId () { return MAPFCLASS_CMD; }

boolean MapFClassCmd::IsA (ClassId id) {
    return MAPFCLASS_CMD == id || Command::IsA(id);
}

MapFClassCmd::MapFClassCmd (ControlInfo* c) : Command(c) { }
MapFClassCmd::MapFClassCmd (Editor* ed) : Command(ed) { }

void MapFClassCmd::Init() {
    _namelist = new StringList;
    _desclist = new StringList;
    String* str_i;
    MapEditor* editor = (MapEditor*)GetEditor();
    int nfcs;
    MapLibState* libstate = editor->libstate();
    MapCovState* covstate = editor->covstate();
    if (libstate->maplib() && strlen(libstate->maplib())>0 &&
	covstate->mapcov() && strlen(covstate->mapcov())>0) {
	MapCoverage* mapcov = (MapCoverage*) editor->GetMapDatabase()->
	    library(libstate->maplib())->coverage(covstate->mapcov());
	int nfcls = mapcov->nfcls();
	for (int i=0; i<nfcls; i++) {
	    str_i = new String(mapcov->feature_class(i)->name());
	    _namelist->append(*str_i);
	    str_i = new String(mapcov->feature_class(i)->description());
	    _desclist->append(*str_i);
	}
    }
    _chooser = new StrChooser(_desclist, new String("Select MAP Feature Class:"),
			      WidgetKit::instance(),
			      Session::instance()->style());
    Resource::ref(_chooser);
}

void MapFClassCmd::Clean() {
    delete _namelist;
    delete _desclist;
    Resource::unref(_chooser);
}

Command* MapFClassCmd::Copy () {
    Command* copy = new MapFClassCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapFClassCmd::Execute () {
    Init();
    MapEditor* editor = (MapEditor*)GetEditor();
    while (_chooser->post_for(editor->GetWindow())) {
        String str;
	String dstr;
        if (_chooser->selected() > -1) {
            str = _namelist->item_ref(_chooser->selected());
	    dstr = _desclist->item_ref(_chooser->selected());
	}
	else
	    continue;
        const char* name = str.string();
	const char* desc = dstr.string();
	editor->fclassstate()->mapfclass(name,desc);
	break;
    }
    Clean();
}

boolean MapFClassCmd::Reversible () { return false; }

/******************************************************************************/

ClassId MapSelectionCmd::GetClassId () { return MAPSELECTION_CMD; }

boolean MapSelectionCmd::IsA (ClassId id) {
    return MAPSELECTION_CMD == id || Command::IsA(id);
}

MapSelectionCmd::MapSelectionCmd (ControlInfo* c) : Command(c) { sel = nil; }
MapSelectionCmd::MapSelectionCmd (Editor* ed) : Command(ed) { sel = nil; }

Command* MapSelectionCmd::Copy () {
    Command* copy = new MapSelectionCmd(CopyControlInfo());
    InitCopy(copy);
    ((MapSelectionCmd*)copy)->Selection(sel);
    return copy;
}

void MapSelectionCmd::Selection (char* s) {
    sel = s;
}

void MapSelectionCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    editor->selstate()->mapselection(sel);
}

boolean MapSelectionCmd::Reversible () { return false; }

/*****************************************************************************/

ClassId MapUpdateMenusCmd::GetClassId () { return MAPUPDATEMENUS_CMD; }

boolean MapUpdateMenusCmd::IsA (ClassId id) {
    return MAPUPDATEMENUS_CMD == id || Command::IsA(id);
}

MapUpdateMenusCmd::MapUpdateMenusCmd (ControlInfo* c)
: Command(c)
{
}

MapUpdateMenusCmd::MapUpdateMenusCmd (Editor* ed)
: Command(ed)
{
}

Command* MapUpdateMenusCmd::Copy () {
    Command* copy = new MapUpdateMenusCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapUpdateMenusCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    if (editor) {
      editor->UpdateLibraryMenu();
      editor->UpdateLayerMenu();
    }
}

boolean MapUpdateMenusCmd::Reversible () { return false; }
